#include <config.h>

#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include "package-setup.h"
#include "parse-info.h"
#include "parse-options.h"
#include "setup-quit.h"
#include "setup-upgrade.h"


/* ===== EXTERN ===== */
int setup_install();
int setup_delete();


// Global structure with options
TSetupOptions gostOptions;
TPackageInfo  gostPackageInfo;

int main( int argc, char *argv[] ) {

  // Parse Options
  bzero(&gostOptions,sizeof(TSetupOptions));
  parse_options( &argc, &argv);
  debug_options();

  // Print Banner
  if ( gostOptions.noheader == 0 ) {
    printTITLE("Bad Penguin - %s_%s by AGX \n", PACKAGE, VERSION );
    normal_color();
  }

  // Link signals
  if (gostOptions.verbose>2) printf("- registering signals\n");
  signal( SIGTERM, setup_quit );
  signal( SIGINT,  setup_quit );
  signal( SIGQUIT, setup_quit );
  signal( SIGKILL, setup_quit );
  signal( SIGHUP,  setup_quit );

  // Perform an action
  switch (gostOptions.action) {
    case ACTION_INSTALL:
    case ACTION_RESTORE:
    case ACTION_UPDATE:
    {
      setup_install();
      break;
    }
    case ACTION_UPGRADE:
    case ACTION_REQUEST:
    {
      setup_upgrade();
      break;
    }
    case ACTION_REMOVE:
    case ACTION_CLEANUP:
    {
      setup_delete();
      break;
    }
    default: {
      printERROR( "! modalita' %i non e' ancora supportata.\n", gostOptions.action );
      normal_color();
      setup_quit(ERROR_PARAMETER);    
    }
  }
  
  if (gostOptions.verbose>2) printf("- %s completed.\n", PACKAGE );
  setup_quit(0);
  return 0;
}
