#include <fstab.h>
#include <mntent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include <badpenguin-chain.h>
#include <badpenguin-fs.h>

#include "parse-info.h"
#include "parse-options.h"
#include "setup-quit.h"

struct stFsSize {
    long long space_free;
    long long space_used;
};

typedef struct stFsSize TFsSize ;

// Decodifica il file INFO con le informazioni sul pacchetto
int parse_diskusages(){
  // vars to parse fstab
  struct mntent *stmnt;
  char mtab_target[256];
  FILE *fdtab;
  // chain
  TChain     *fschain;
  TChainItem *fsItem;
  TFsSize    *fsSize;
  // diskusages
  char *p = NULL;
  char *l = NULL;
  char dir_size[255];
  char dir_name[255];
  int i;
  long long diff;
    
  // go
  if (gostOptions.verbose>0) printf("- checking diskusages \n");    
  // prepare fstab name
  strcpy( mtab_target, gostOptions.targetdir );
  strcat( mtab_target, "/etc/fstab" );
  if (gostOptions.verbose>2) printf("fstab file: %s \n", mtab_target );
  // read
  fschain = chain_create();
  fdtab = setmntent( mtab_target, "r" );
  stmnt = getmntent(fdtab);
  while ( stmnt != NULL ) {
   //printf( "? %s \n", stmnt->mnt_dir );
   // insert and add name
   if (stmnt->mnt_dir[0] == '/') {
     fsItem = chain_insert(fschain, stmnt->mnt_dir, 1 );
     fsItem->name = strdup(stmnt->mnt_dir);
     //printf( "  count = %i \n\n", fschain->count );
     // get free space and add data
     fsSize = calloc( 1, sizeof(TFsSize));
     strcpy( dir_name, gostOptions.targetdir );
     strcat( dir_name, "/" );
     strcat( dir_name, fsItem->name );
     fsSize->space_free = diskfree(dir_name);
     fsItem->data = fsSize;
   }
   // next
   stmnt = getmntent(fdtab);
  }
  endmntent(fdtab);
  
  // for safety append at the end the "/" dir
  fsItem = chain_append(fschain);
  fsItem->name = "/";
  fsSize = calloc( 1, sizeof(TFsSize));
  fsSize->space_free = diskfree(gostOptions.targetdir);
  fsItem->data = fsSize;  
  
  // print chain
  #ifdef DEBUG
  printf("Printing chain ........\n");
  fsItem = fschain->head;
  while (fsItem!=NULL) {
    printf( "> %s", fsItem->name );
    if (fsItem->data!=NULL)
      printf( " = %lli - %lli", 
        ((TFsSize*)(fsItem->data))->space_free, 
        ((TFsSize*)(fsItem->data))->space_used
      );
    printf("\n");
    //
    fsItem = fsItem -> next;
  }
  #endif
  
  // Parsing diskusages
  p = gostPackageInfo.diskusages;  
  //p = "243856943:usr  387538:usr/local 385748:usr/share     38548435:usr/info  611111000:usr/src  50000000:usr/dict 7000000:usr/share/bpp 700000:pippo";
  while (p[0]!=0) {
    // skip blanks  
    while (p[0]==' ') p++;
    // search for ':'    
    l=p;
    while ((l[0]!=':')&&(l[0]!=0)) l++;
    if (l[0]==0) break;
    i=l-p;
    //printf("- trovato1 : a lunghezza %i da %s \n", i, p );
    strncpy( dir_size, p, i );
    dir_size[i]=0;
    p=l+1;
    l=p;
    // search for ' '
    while ((l[0]!=' ')&&(l[0]!=0)) l++;
    i=l-p;
    //printf("- trovato2 : a lunghezza %i da %s \n", i, p );
    strcpy( dir_name, gostOptions.targetdir);    
    strncpy( dir_name+strlen(gostOptions.targetdir), p, i );
    dir_name[ strlen(gostOptions.targetdir) + i ]=0;
    if (l[0]!=0) {
      p=l+1;
      l=p;
    }    
    // cerca directory
    if (gostOptions.verbose>2) printf("- verifica : %s - %s \n", dir_size, dir_name );
    fsItem = fschain->head;
    while (fsItem!=NULL) {
      if ( 0 == strncmp( fsItem->name, dir_name, strlen(fsItem->name) )) {
        //printf("- trovato : %s <== %s \n", fsItem->name, dir_name );
        ((TFsSize*)(fsItem->data))->space_used += atoll(dir_size);
        //printf("- used: %lli \n", ((TFsSize*)(fsItem->data))->space_used );
	diff = ((TFsSize*)(fsItem->data))->space_free - ((TFsSize*)(fsItem->data))->space_used;
	if (gostOptions.verbose>1) printf( "- space left on %s is %lliK\n", fsItem->name, diff/1024 );
	if ( (diff<10000000) && (diff>10240) ) {
	  printWARNING("- low disk space on %s <== %s , %lliK lefts\n", fsItem->name, dir_name, diff/1024 );
	  normal_color();
	} else
	if (diff<10240) {
	  printERROR( "- disk full on %s <== %s \n", fsItem->name, dir_name );
	  normal_color();
	  setup_quit(ERROR_IO);
	}
	break;
      }
      // next
      fsItem = fsItem -> next;
    }
  }
  return 0;
}
