#include <sys/utsname.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "badpenguin-ansi.h"
#include "badpenguin-fs.h"
#include "badpenguin-errors.h"
#include "parse-info.h"
#include "parse-options.h"
#include "setup-quit.h"

// Tokens
#define TSONAME      "SONAME="
#define LSONAME      strlen(TSONAME)
#define TVERSION     "VERSION="
#define LVERSION     strlen(TVERSION)
#define TCATEGORY    "CATEGORY="
#define LCATEGORY    strlen(TCATEGORY)
#define TDISKNAME    "DISKNAME="
#define LDISKNAME    strlen(TDISKNAME)
#define TARCH        "ARCH="
#define LARCH        strlen(TARCH)
#define TSUMMARY     "SUMMARY="
#define LSUMMARY     strlen(TSUMMARY)
#define TMANTAINER   "MANTAINER="
#define LMANTAINER   strlen(TMANTAINER)
#define TOSNAME      "OSNAME="
#define LOSNAME      strlen(TOSNAME)
#define TCONFLICTS   "CONFLICTS="
#define LCONFLICTS   strlen(TCONFLICTS)
#define TDISKUSAGES  "DISKUSAGES="
#define LDISKUSAGES  strlen(TDISKUSAGES)
#define TDISKUSAGE   "DISKUSAGE="
#define LDISKUSAGE   strlen(TDISKUSAGE)
#define TREQUEST     "REQUEST="
#define LREQUEST     strlen(TREQUEST)



// Copia saltando skip caratteri e i doppi apici
char * copy_token( char *source, int skip ) {
  char *target = NULL;
  int start, end, len;
  //printf("copy_token: %s \n", source );
  start=skip;
  while ((source[start]=='"')||(source[start]==' ')||(source[start]=='\'')||(source[start]==7)||(source[start]==9)) start++;
  end = strlen(source);
  while ((source[end]=='"')||(source[end]==' ')||(source[end]=='\'')||(source[end]==7)||(source[end]==9)||(source[end]==0)) end--;
  len = end-start+1;
  if (len<=0) return strdup("");
  //if (len>0) len--; 
  target = (char*) calloc( len+1, sizeof(char) ); 
  strncpy( target, source + start, len );
  target[len]=0;
  return target;
}


// Decodifica il file INFO con le informazioni sul pacchetto
int parse_info(){
  FILE *fd;
  // badpenguin-filesystem package has a very big diskusages !
  char line[1024];
  struct utsname utstemp;

  if (gostOptions.verbose>0) printf("- parsing package's INFO\n");

  // Open INFO
  line[0] = 0;
  strcat(line, gostOptions.tempdir);
  strcat(line, "/install/INFO");
  bzero( &gostPackageInfo, sizeof(TPackageInfo) );
  
  // Apre il file INFO    
  fd = fopen(line, "r");
  if (fd == NULL) {
    printERROR("! impossibile aprire il file %s !\n", line );
    normal_color();
    setup_quit(ERROR_FORMAT);
  }
  
  // read
  while ( ! feof(fd) ) {
    if ( 0 == read_line( fd, line)) continue;
    // SONAME
    if ( 0 == strncmp( TSONAME, line, LSONAME ) ) {
      gostPackageInfo.soname = copy_token(line, LSONAME );
      if (gostOptions.verbose>2) printf("-     SONAME = %s \n", gostPackageInfo.soname );
    } else
    if ( 0 == strncmp( TVERSION, line, LVERSION ) ) {
      gostPackageInfo.version = copy_token(line, LVERSION );
      if (gostOptions.verbose>2) printf("-    VERSION = %s \n", gostPackageInfo.version );
    } else
    if ( 0 == strncmp( TCATEGORY, line, LCATEGORY ) ) {
      gostPackageInfo.category = copy_token(line, LCATEGORY );
      if (gostOptions.verbose>2) printf("-   CATEGORY = %s \n", gostPackageInfo.category );
    } else
    if ( 0 == strncmp( TDISKNAME,line, LDISKNAME ) ) {
      gostPackageInfo.diskname = copy_token(line, LDISKNAME );
      if (gostOptions.verbose>2) printf("-   DISKNAME = %s \n", gostPackageInfo.diskname );
    } else
    if ( 0 == strncmp( TARCH, line, LARCH ) ) {
      gostPackageInfo.arch = copy_token(line, LARCH );
      if (gostOptions.verbose>2) printf("-       ARCH = %s \n", gostPackageInfo.arch );
    } else
    if ( 0 == strncmp( TSUMMARY, line, LSUMMARY ) ) {
      gostPackageInfo.summary = copy_token(line, LSUMMARY );
      if (gostOptions.verbose>2) printf("-    SUMMARY = %s \n", gostPackageInfo.summary );
    } else
    if ( 0 == strncmp( TMANTAINER, line, LMANTAINER ) ) {
      gostPackageInfo.mantainer = copy_token(line, LMANTAINER );
      if (gostOptions.verbose>2) printf("-  MANTAINER = %s \n", gostPackageInfo.mantainer );
    } else
    if ( 0 == strncmp( TOSNAME, line, LOSNAME ) ) {
      gostPackageInfo.osname = copy_token(line, LOSNAME );
      if (gostOptions.verbose>2) printf("-     OSNAME = %s \n", gostPackageInfo.osname );
    } else
    if ( 0 == strncmp( TCONFLICTS, line, LCONFLICTS ) ) {
      gostPackageInfo.conflicts = copy_token(line, LCONFLICTS );
      if (gostOptions.verbose>2) printf("-  CONFLICTS = %s \n", gostPackageInfo.conflicts );
    } else
    if ( 0 == strncmp( TDISKUSAGE, line, LDISKUSAGE ) ) {
      gostPackageInfo.diskusage = atol( copy_token(line, LDISKUSAGE ));
      if (gostOptions.verbose>2) printf("- DISKUSAGE = %lli \n", gostPackageInfo.diskusage );
    } else
    if ( 0 == strncmp( TDISKUSAGES, line, LDISKUSAGES ) ) {
      gostPackageInfo.diskusages = copy_token(line, LDISKUSAGES );
      if (gostOptions.verbose>2) printf("- DISKUSAGES = %s \n", gostPackageInfo.diskusages );
    } else
    if ( 0 == strncmp( TREQUEST, line, LREQUEST ) ) {
      gostPackageInfo.request = copy_token(line, LREQUEST );
      if (gostOptions.verbose>2) printf("-    REQUEST = %s \n", gostPackageInfo.request );
    }    
  // end while      
  }
  fclose(fd);

  // Checking params
  if((gostPackageInfo.soname==NULL)||(gostPackageInfo.version==NULL)) {
    printERROR("! the SONAME or the VERSION field is null\n");
    normal_color();
    setup_quit(ERROR_FORMAT);
  }
  if((strlen(gostPackageInfo.soname)==0)||(strlen(gostPackageInfo.version)==0)) {
    printERROR("! the SONAME or the VERSION field is empty\n");
    normal_color();
    setup_quit(ERROR_FORMAT);
  } 
  
  // Assemble fullname
  gostPackageInfo.fullname = calloc(256, sizeof(char));
  assert(gostPackageInfo.fullname);
  gostPackageInfo.fullname[0] = 0;
  strcat( gostPackageInfo.fullname,  gostPackageInfo.soname );
  strcat( gostPackageInfo.fullname,  "_" );
  strcat( gostPackageInfo.fullname,  gostPackageInfo.version );
  if (gostOptions.verbose>1) printf("- fullversion: %s \n", gostPackageInfo.fullname );
  
  // check for Arch & OS
  if (gostOptions.verbose>1) printf("- checking arch and OS\n");
  uname(&utstemp);
  if (gostOptions.verbose>2) printf("- sysname: %s vs %s \n", utstemp.sysname, gostPackageInfo.osname );
  if ( 0 != strcasecmp(utstemp.sysname, gostPackageInfo.osname )) {
    printERROR("! you cannot install a package for '%s' under '%s'\n", gostPackageInfo.osname, utstemp.sysname);
    normal_color();
    return (ERROR_FORMAT);
  }  
  if ( 0 != strcasecmp("ANY", gostPackageInfo.arch)) {
    if (gostOptions.verbose>2) printf("- machine: %s vs %s\n", utstemp.machine, gostPackageInfo.arch );
    // General compare
    if ( 0 != strcasecmp(utstemp.machine, gostPackageInfo.arch)) {
      // try to guess i386<>i586
      if ((utstemp.machine[0] == 'i') && (gostPackageInfo.arch[0] == 'i')) {
        if ( utstemp.machine[1] < gostPackageInfo.arch[1] ) {
          printERROR("! you cannot install an intel package for '%s' under '%s'\n", gostPackageInfo.arch, utstemp.machine);
          normal_color();      
          return (ERROR_FORMAT);
	}
      } else {
        printERROR("! you cannot install a package for '%s' under '%s'\n", gostPackageInfo.arch, utstemp.machine);
        normal_color();      
        return (ERROR_FORMAT);
      }
    }
  }
  // quit
  return 0;
}
