#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include "package-setup.h"
#include "parse-options.h"
#include "setup-help.h"
#include "setup-quit.h"


// Analizza e ricava i parametri della riga di comando di setup
// riempiendo la struttura delle opzioni
int parse_options( int *lpi, char ***lpac) {
  int i;

  // Se non ci sono sufficienti parametri richiama la guida in linea
  if ( *lpi < 2 ) {
    printERROR("! numero insufficiente di parametri.\n" );
    normal_color();
    setup_help();
  }
  
  // Analisi parametri
  bzero( &gostOptions, sizeof(TSetupOptions) );
  for ( i=1; i<*lpi; i++ ) {
    if (gostOptions.verbose>2) printf("- parametro nr. %i : %s \n", i, (*lpac)[i] );
    /* Installazione */
    if ( 0 == strcmp( "-i", (*lpac)[i]) )  {
      gostOptions.action = ACTION_INSTALL;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_INSTALL\n");
    } else
    if ( 0 == strcmp( "--install", (*lpac)[i]) )  {
      gostOptions.action = ACTION_INSTALL;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_INSTALL\n");
    } else
    /* restore & update */
    if ( 0 == strcmp( "--restore", (*lpac)[i]) )  {
      gostOptions.action = ACTION_RESTORE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_RESTORE\n");
    } else    
    if ( 0 == strcmp( "--update", (*lpac)[i]) )  {
      gostOptions.action = ACTION_UPDATE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_UPDATE\n");
    } else    
    /* upGrade */
    if ( 0 == strcmp( "--upgrade", (*lpac)[i]) )  {
      gostOptions.action = ACTION_UPGRADE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_UPGRADE\n");
    } else    
    if ( 0 == strcmp( "--request", (*lpac)[i]) )  {
      gostOptions.action = ACTION_REQUEST;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_UPGRADE\n");
    } else    
    /* Rimozione */
    if ( 0 == strcmp( "-e", (*lpac)[i]) )  {
      gostOptions.action = ACTION_REMOVE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_REMOVE\n");
    } else
    if ( 0 == strcmp( "--remove", (*lpac)[i]) )  {
      gostOptions.action = ACTION_REMOVE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_REMOVE\n");
    } else
    if ( 0 == strcmp( "--delete", (*lpac)[i]) )  {
      gostOptions.action = ACTION_REMOVE;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_REMOVE\n");
    } else
    if ( 0 == strcmp( "--cleanup", (*lpac)[i]) )  {
      gostOptions.action = ACTION_CLEANUP;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_CLEANUP\n");
    } else
    if ( 0 == strcmp( "--purge", (*lpac)[i]) )  {
      gostOptions.action = ACTION_CLEANUP;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_CLEANUP\n");
    } else
    if ( 0 == strcmp( "--prune", (*lpac)[i]) )  {
      gostOptions.action = ACTION_CLEANUP;
      gostOptions.interactive=0;
      if (gostOptions.verbose>2) printf("- parametro ACTION_CLEANUP\n");
    } else
    /* OPTIONS */
    if ( 0 == strcmp( "-h", (*lpac)[i]) ) 
      setup_help();
    else
    if ( 0 == strcmp( "--help", (*lpac)[i]) ) 
      setup_help();
    /* Opzioni */
    else
    if ( 0 == strcmp( "--no-header", (*lpac)[i]) ) {
      gostOptions.noheader=1;
      if (gostOptions.verbose>2) printf("- parametro NOHEADER\n");
    } else
    if ( 0 == strcmp( "-v", (*lpac)[i]) ) {
      gostOptions.verbose++;
      if (gostOptions.verbose>2) printf("- parametro VERBOSE\n");
    } else
    if ( 0 == strcmp( "--verbose", (*lpac)[i]) ) {
      gostOptions.verbose++; 
      if (gostOptions.verbose>2) printf("- parametro VERBOSE\n");
    } else 
    if ( 0 == strcmp( "--debug", (*lpac)[i]) ) {
      gostOptions.debug=1;
      if (gostOptions.verbose>2) printf("- parametro DEBUG\n");
    } else 
    /* Default: probabilmente un PARAMETER esteso */
    {
      // Controlla se e' un 'installazione interattiva
      if (( (*lpac)[i][0] != '-' ) && ( gostOptions.sourcefile == NULL )) {
        gostOptions.sourcefile = strdup( (*lpac)[i] );
        if (gostOptions.verbose>2) printf("- pacchetto da installare: %s \n", gostOptions.sourcefile );	
        if ( gostOptions.action == ACTION_UNKNOWN ) {
          if (gostOptions.verbose>1) printf("- seleziono installazione interattiva \n" );	
          gostOptions.action = ACTION_INSTALL;
          gostOptions.targetdir = strdup("/");
	  gostOptions.interactive=1;
	} else {
	  if (gostOptions.verbose > 1 ) {
            printWARNING("- seleziono l'installazione non interattiva \n" );
	    normal_color();
	  }
	  gostOptions.interactive=0;
	}
      } else
      // Se il nome del pacchetto e' stato gia inserito si tratta di una dir
      if (( (*lpac)[i][0] != '-' ) && ( gostOptions.targetdir == NULL )) {
        gostOptions.targetdir = strdup( (*lpac)[i] );
        if (gostOptions.verbose>2) printf("- directory d'installazione: %s \n", gostOptions.targetdir );		
      } else {
        printERROR("! parametro (%i) [%s] non riconosciuto.\n", i, (*lpac)[i] );
	normal_color();
        setup_quit ( ERROR_PARAMETER );
      }
      // end of default
    }
  // next i
  }  
  // Ok
  return 0;
}


// Stampa il debug delle opzioni
int debug_options() {
  if ( gostOptions.verbose<3) return 0;
  printf( "\n" );
  printf( "[  Debug Header  ] \n");
  printf( "Source File .....: %s\n", gostOptions.sourcefile );
  printf( "\n" );
  printf( "Target Dir ......: %s\n", gostOptions.targetdir );
  printf( "\n" );
  printf( "Action ..........: %i\n", gostOptions.action );
  printf( "No Header .......: %i\n", gostOptions.noheader );
  printf( "Interactive .....: %i\n", gostOptions.interactive );
  printf( "Relaxed .........: %i\n", gostOptions.relaxed );
  printf( "Spool Root ......: %s\n", gostOptions.spoolroot );
  printf( "Spool Dir .......: %s\n", gostOptions.spooldir );
  printf( "\n" );
  printf( "Verbose .........: %i\n", gostOptions.verbose );
  printf( "\n");
  return 0;
}
