#include <sys/stat.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-chain.h>
#include <badpenguin-errors.h>
#include <badpenguin-fs.h>
#include <badpenguin-untar.h>
#include "package-setup.h"
#include "parse-conflicts.h"
#include "parse-diskusages.h"
#include "parse-info.h"
#include "parse-options.h"
#include "parse-request.h"
#include "setup-quit.h"
#include "setup-script.h"


// Installa, Aggiorna, Ripristina il pacchetto
int setup_delete() {
  char tempspool[255], tempmanifest[255];
  FILE *fd = NULL;
  char buffer[255], tempfile[255], tempfile2[255];
  TChain     *dirchain = NULL;
  TChainItem *diritem  = NULL;
  int iret = 0;
  
  if (gostOptions.verbose>2) printf("- inizio rimozione pacchetto\n");

  // Nome del pacchetto ?
  if ( gostOptions.sourcefile == NULL ) {
    printERROR("! a package name was not specified\n");
    normal_color();
    setup_quit(ERROR_PARAMETER);
  }

  // Target dir ?
  if ( gostOptions.targetdir == NULL ) {
    printWARNING("! target dir is null then i assume interactive remotion on '/'\n");
    normal_color();
    gostOptions.targetdir = strdup("/");
    gostOptions.interactive = 1;
  }

  // calculate dir & filename
  sprintf( tempspool,   "%s%s/%s", gostOptions.targetdir, DEFAULT_SPOOLROOT, gostOptions.sourcefile);
  sprintf( tempmanifest, "%s/MANIFEST", tempspool);  
  gostOptions.tempscript = calloc( 255, sizeof(char));
  sprintf( gostOptions.tempscript, "%s/scripts", tempspool);  
  
  // package should be already installed
  if ( 0 != file_exists(tempspool) ) {
    printERROR("! pacchetto %s non registrato \n", gostOptions.sourcefile);
    normal_color();
    setup_quit(ERROR_IO);
  }
  // ... and MANIFEST present
  if ( 0 != file_exists(tempmanifest) ) {
    printERROR("! MANIFEST non presente\n");
    normal_color();
    setup_quit(ERROR_IO);
  }  

  // Print Banner
  if (gostOptions.action == ACTION_CLEANUP) {
    printf( "%s> cleaning %s%s\n", FG_WHITE, FG_GREEN, gostOptions.sourcefile );
  } else {
    printf( "%s> removing %s%s\n", FG_WHITE, FG_GREEN, gostOptions.sourcefile );
  }
  normal_color();

  // run PRE scritps
  setup_script(gostOptions.tempscript, "PRE-REMOVE", "script di Pre Remove");
  setup_script(gostOptions.tempscript, "PRE-DELETE", "script di Pre Remove");

  // Apre il MANIFEST
  fd = fopen( tempmanifest, "r");
  if (fd==NULL) {
    printERROR("! errore aprendo il MANIFEST in lettura !\n");
    normal_color();
    return ERROR_IO;
  }
  // Legge l`elenco dei files
  while (! feof(fd) ) {
    read_line( fd, buffer );
    // Ignora install e righe vuote
    if ( 0 == strncmp(buffer, "install/", 8) ) continue;
    if ( 0 == strlen(buffer) ) continue;
    // Rimuove gli incoming in caso di purge altrimenti li ignora
    if ( 0 == strncmp(buffer, "incoming/", 9) ){
      if ( ACTION_PURGE == gostOptions.action ) {
        strcpy(buffer, buffer+9);
	printf("purge: %s\n", buffer);
      } else {
        continue;
      }
    }    
    // Aggiunge al nome del file la target dir !
    sprintf(tempfile, "%s/%s", gostOptions.targetdir, buffer );    
    // Cancella il file
    iret = unlink(tempfile);
    if (iret==0) {
      if (gostOptions.verbose>2) printf( "> rm file %s\n", tempfile );
    } else {    
      // If file has not been removed is because it is a DIR
      // so we cache it for later use
      if (dirchain==NULL) dirchain = chain_create();
      diritem = chain_append(dirchain);
      diritem->name = strdup(tempfile);
    }
  }
  fclose(fd);

  // Rimuove ogni directory nella cashe secondo l'ordine tail-->head
  // OPS! Solo se e' vuota la proc cerca di eliminare anche i rami principali
  if (dirchain!=NULL) {
    //printf("Cached dirs: %i \n", dirchain->count );
    diritem = dirchain->tail;
    while (diritem!=NULL) {
      //printf("DIR: %s", diritem->name);
      iret = rmdir( diritem->name );
      if (iret==0) {
        if (gostOptions.verbose>2) printf("> rm dir  %s\n", diritem->name);
      }
      //printf( " = %i \n", iret );      
      diritem=diritem->prev;
    }
    // Clean-up cache
    chain_delete(dirchain);
  }

  // run DELETE scritps
  setup_script(gostOptions.tempscript, "REMOVE", "script di Remove");
  setup_script(gostOptions.tempscript, "DELETE", "script di Remove");

  // run CLEANUP scritps
  if ( ACTION_PURGE == gostOptions.action ) {  
    setup_script(gostOptions.tempscript, "CLEANUP", "script di Clean Up");  
    setup_script(gostOptions.tempscript, "PURGE",   "script di Clean Up");
    setup_script(gostOptions.tempscript, "PRUNE",   "script di Clean Up");
  }
  
  // Schedule POST-REMOVE script
  sprintf( tempfile, "%s/POST-REMOVE", gostOptions.tempscript );
  if ( 0 == file_exists(tempfile) ) {
    if (gostOptions.verbose > 0 ) printf( "- schedulazione dello script di POST-REMOVE\n" );
    sprintf( tempfile2, "%s/%s.post-remove", gostOptions.spoolroot, gostPackageInfo.fullname );
    if ( 0 != move_file(tempfile, tempfile2 ) ) {
      printERROR("! errore durante lo spostamento dello script di POST-REMOVE\n");
      normal_color();
      setup_quit(ERROR_IO);
    }
  }
  // Schedule POST-DELETE script
  sprintf( tempfile, "%s/POST-DELETE", gostOptions.tempscript );
  if ( 0 == file_exists(tempfile) ) {
    if (gostOptions.verbose > 0 ) printf( "- schedulazione dello script di POST-REMOVE\n" );
    sprintf( tempfile2, "%s/%s.post-remove", gostOptions.spoolroot, gostPackageInfo.fullname );
    if ( 0 != move_file(tempfile, tempfile2 ) ) {
      printERROR("! errore durante lo spostamento dello script di POST-REMOVE\n");
      normal_color();
      setup_quit(ERROR_IO);
    }
  }
  
  // Unregistra il pacchetto
  if (gostOptions.verbose > 0 ) printf( "- deregistrazione pacchetto\n" );  
  deltree(tempspool);

  // Well done, call finalize on interactive installation
  printf( "- rimozione pacchetto completata con successo\n" );
  return 0;
}
