#include <sys/stat.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include <badpenguin-fs.h>
#include <badpenguin-untar.h>
#include "package-setup.h"
#include "parse-conflicts.h"
#include "parse-diskusages.h"
#include "parse-info.h"
#include "parse-options.h"
#include "parse-request.h"
#include "setup-quit.h"
#include "setup-script.h"
#include "setup-upgrade.h"

// Installa, Aggiorna, Ripristina il pacchetto
int setup_install() {
  char temp1[255], temp2[255];
  char lostTemplate[10];
  TTarOptions lostTarOpt;
  int retval;
  long long lifspace;

  if (gostOptions.verbose>2) printf("- inizio installazione pacchetto\n");

  // Source file name - not null
  if ( gostOptions.sourcefile == NULL ) {
    printERROR("! a filename for the package was not specified\n");
    normal_color();
    setup_quit(ERROR_PARAMETER);
  }
  // Target dir - not null
  if ( gostOptions.targetdir == NULL ) {
    printWARNING("! target dir is null then i assume interactive installation on /\n");
    normal_color();
    gostOptions.targetdir = strdup("/");
    gostOptions.interactive = 1;
  }

  // check if it is a package name or a filename
  if (gostOptions.action != ACTION_RESTORE ) {
    if ( NULL == strstr( gostOptions.sourcefile, ".bpp" ) ) {
      // Ok, this is not a package so goes to get it    
      return setup_upgrade();
    }
  
    // If package's name DOES NOT begins with a slash
    // then we have to add the current dir in front of it.
    if ( gostOptions.sourcefile[0] != '/' ) {
      //strcpy( temp1, getcwd( NULL, 255 ) );
      getcwd( temp1, 255 );
      assert(temp1 != NULL);
      strcat( temp1, "/" );
      strcat( temp1, gostOptions.sourcefile );
      if (gostOptions.sourcefile!=NULL) free( gostOptions.sourcefile );
      gostOptions.sourcefile = strdup(temp1);
    }
  }
  
  // The filename NOW should exist !  
  if ( 0 != file_exists(gostOptions.sourcefile) ) {
    printERROR("! the package file '%s' does not exists\n", gostOptions.sourcefile );
    normal_color();
    setup_quit(ERROR_NOTFOUND);
  }

  if (gostOptions.verbose>2) printf("- package %s exist\n", gostOptions.sourcefile );

  if (gostOptions.action != ACTION_RESTORE ) {
    // alloc tempdir
    gostOptions.tempdir        = calloc(255,1);
    assert(gostOptions.tempdir!=NULL);
    if ( 0 != strcmp(gostOptions.targetdir,"/") ) {
      strcpy(gostOptions.tempdir, gostOptions.targetdir);
      strcat(gostOptions.tempdir, DEFAULT_TEMPROOT );
    } else {
      strcpy(gostOptions.tempdir, DEFAULT_TEMPROOT);  
    }
    strcat(gostOptions.tempdir, "/" );
    strcpy( lostTemplate, "XXXXXX\0");
    mktemp( lostTemplate );
    strcat(gostOptions.tempdir, lostTemplate );
    if (gostOptions.verbose>2) printf( "- template: %s\n", gostOptions.tempdir );  

    // Check FreeSpace    
    if ( 0 != strcmp(gostOptions.targetdir,"/") ) {
      strcpy(temp1, gostOptions.targetdir);
      strcat(temp1, DEFAULT_TEMPROOT );
    } else {
      strcpy(temp1, DEFAULT_TEMPROOT);  
    }
    lifspace = diskfree(temp1);
    if (lifspace<0) setup_quit(ERROR_IO);
    if (lifspace< (500*1024) ) {
      printERROR("! not enough free space (%lli) under %s\n", lifspace, temp1);
      normal_color();
      setup_quit(ERROR_IO);
    }
  
    // Untar solo la directory install del pacchetto
    if (gostOptions.verbose>0) printf( "- estrazione informazioni di installazione dal pacchetto\n" );
    lostTarOpt.sourcefile = strdup(gostOptions.sourcefile);
    lostTarOpt.targetdir  = strdup(gostOptions.targetdir);
    lostTarOpt.prefixdir  = strdup(gostOptions.tempdir);
    lostTarOpt.mode       = TAR_MODE_HEADER;
    lostTarOpt.action     = TAR_ACTION_INSTALL;
    lostTarOpt.verbose    = gostOptions.verbose;
    lostTarOpt.maxsize    = 0;
    retval = untar( &lostTarOpt );
    if (retval != 0 ) setup_quit(ERROR_IO);

    // check if /install directory is there
    strcpy( temp1, gostOptions.tempdir );
    strcat( temp1, "/install" );
    if ( 0 != file_exists(temp1) ) {
      printERROR("! directory /install non trovata in %s\n", temp1);
      normal_color();
      setup_quit(ERROR_IO);
    }

    // Parse Info
    parse_info();
  }
  
  // allocate spoolroot, spooldir, tempscript
  gostOptions.spoolroot      = calloc(255,sizeof(char));
  strcpy(gostOptions.spoolroot, gostOptions.targetdir );
  strcat(gostOptions.spoolroot, DEFAULT_SPOOLROOT );
  gostOptions.spooldir       = calloc(255,sizeof(char));
  strcpy(gostOptions.spooldir, gostOptions.spoolroot );
  strcat(gostOptions.spooldir, "/" );
  strcat(gostOptions.spooldir, gostPackageInfo.fullname );
  gostOptions.tempscript     = calloc(255,sizeof(char));
  strcpy(gostOptions.tempscript, gostOptions.tempdir );
  strcat(gostOptions.tempscript, "/install/scripts" );

  // package should be already installed
  if (
    (gostOptions.action == ACTION_RESTORE ) ||
    (gostOptions.action == ACTION_UPDATE )
  ) {
    if ( 0 != file_exists(gostOptions.spooldir) ) {
      printERROR("! pacchetto %s non registrato \n", gostOptions.sourcefile);
      normal_color();
      setup_quit(ERROR_IO);
    }
  }
  
  // Print Info
  if (gostOptions.action == ACTION_RESTORE ) {
    printf(
      "%s> restoring %s%s %s(%s) %s%s%s\n", 
      FG_WHITE, 
      FG_GREEN,   gostPackageInfo.fullname, 
      FG_BLUE,    gostPackageInfo.category,
      FG_MAGENTA, gostPackageInfo.summary,
      FG_WHITE
    );
  } else 
  if (gostOptions.action == ACTION_UPDATE ) {
    printf(
      "%s> updating %s%s %s(%s) %s%s%s\n", 
      FG_WHITE, 
      FG_GREEN,   gostPackageInfo.fullname, 
      FG_BLUE,    gostPackageInfo.category,
      FG_MAGENTA, gostPackageInfo.summary,
      FG_WHITE
    );
  } else {
    printf(
      "%s> installing %s%s %s(%s) %s%s%s\n", 
      FG_WHITE, 
      FG_GREEN,   gostPackageInfo.fullname, 
      FG_BLUE,    gostPackageInfo.category,
      FG_MAGENTA, gostPackageInfo.summary,
      FG_WHITE
    );
  }
  
  
  if (gostOptions.action != ACTION_RESTORE ) {
    if (gostOptions.action != ACTION_UPDATE ) {
      // Parse Free disk space
      parse_diskusages();
      // Parse Dependencies
      parse_request();
      // Parse AGAIN Free disk space after installing deps
      parse_diskusages();
      // Remove Conflicts
      parse_conflicts();  
    }
    // run PRE-INSTALL
    setup_script(gostOptions.tempscript, "PRE-INSTALL", "script Pre Install");
  }
  
  
  // Untar files
  if (gostOptions.verbose>0) printf( "- estrazione files dal pacchetto\n" );
  if (lostTarOpt.sourcefile==NULL) lostTarOpt.sourcefile = strdup(gostOptions.sourcefile);
  if (lostTarOpt.targetdir==NULL) lostTarOpt.targetdir  = strdup(gostOptions.targetdir);

  lostTarOpt.mode       = TAR_MODE_ALL;
  if (gostOptions.action == ACTION_RESTORE ) {
    lostTarOpt.action     = TAR_ACTION_RESTORE;
  } else
  if (gostOptions.action == ACTION_UPDATE ) {
    lostTarOpt.action     = TAR_ACTION_UPDATE;
  } else {
    lostTarOpt.action     = TAR_ACTION_INSTALL;
  }
  lostTarOpt.verbose    = gostOptions.verbose;
  lostTarOpt.prefixdir  = lostTarOpt.targetdir;
  // just set it > 1 ... untar will do the rest
  lostTarOpt.maxsize    = gostPackageInfo.diskusage;
  retval = untar( &lostTarOpt );
  if (retval != 0 ) setup_quit(ERROR_IO);


  // RESTORE quits here
  if (gostOptions.action == ACTION_RESTORE ) {
    printf( "- restore pacchetto completato con successo\n" );  
    return 0;
  }
  
  // ldconfig when in interactive mode (prima di INSTALL)
  if (gostOptions.action != ACTION_UPDATE ) {
    if (gostOptions.interactive == 1) {
      if (gostOptions.verbose>0) printf("- configurazione delle librerie dinamiche ... \n");
        normal_color();
      runshell("ldconfig");
    }
  }
  
  // run INSTALL
  setup_script(gostOptions.tempscript, "INSTALL", "script Install");

  // Schedule POST-INSTALL script
  sprintf( temp1, "%s/POST-INSTALL", gostOptions.tempscript );
  if ( 0 == file_exists(temp1) ) {
    if (gostOptions.verbose > 0 ) printf( "- schedulazione dello script di POST-INSTALL\n" );
    sprintf( temp2, "%s/%s.post-install", gostOptions.spoolroot, gostPackageInfo.fullname );
    if ( 0 != move_file(temp1, temp2 ) ) {
      printERROR("! errore durante lo spostamento dello script di POST-INSTALL\n");
      normal_color();
      setup_quit(ERROR_IO);
    }
  }

  // Schedule X-POST-INSTALL script
  sprintf( temp1, "%s/X-POST-INSTALL", gostOptions.tempscript );
  if ( 0 == file_exists(temp1) ) {
    if (gostOptions.verbose > 0 ) printf( "- schedulazione dello script di X-POST-INSTALL\n" );
    sprintf( temp2, "%s/%s.x-post-install", gostOptions.spoolroot, gostPackageInfo.fullname );
    if ( 0 != move_file(temp1, temp2 ) ) {
      printERROR("! errore durante lo spostamento dello script di X-POST-INSTALL\n");
      normal_color();
      setup_quit(ERROR_IO);
    }
  }

  // UPDATE quits here
  if (gostOptions.action == ACTION_UPDATE ) {
    printf( "- update pacchetto completato con successo\n" );  
    return 0;
  }

  // Registrazione Pacchetto
  if (gostOptions.verbose > 0 ) printf( "- registrazione informazioni pacchetto\n" );
  // clean up old dir if exist
  if ( 0 == file_exists(gostOptions.spooldir) ) {
    deltree(gostOptions.spooldir);
  }
  strcpy(temp1, gostOptions.tempdir );
  strcat(temp1, "/install" );
  if ( 0 != move_dir(temp1, gostOptions.spooldir ) ) {
    printERROR("! errore durante la registrazione delle informazioni sul pacchetto\n");
    normal_color();
    setup_quit(ERROR_IO);
  }

  // Well done, call finalize on interactive installation
  printf( "- installazione pacchetto completata con successo\n" );

  // Run --finalize in interactive mode
  // !TODO!
  if ( gostOptions.interactive == 1 ) {
    gostOptions.noheader = 1;
    gostOptions.action = ACTION_FINALIZE;
    //return setup_finalize();
  }  
  return 0;
}
